import uuid

from furl import furl

from django.db import models
from django.urls import reverse
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from mayan.apps.converter.transformations import BaseTransformation
from mayan.apps.documents.models import Document, DocumentPage
from mayan.apps.events.classes import (
    EventManagerMethodAfter, EventManagerSave
)
from mayan.apps.events.decorators import method_event

from .events import event_shareable_link_created, event_shareable_link_edited


class ShareableLink(models.Model):
    document = models.ForeignKey(
        help_text=_('The document to which to link.'),
        on_delete=models.CASCADE, related_name='shareable_links',
        to=Document, verbose_name=_('Document')
    )
    description = models.TextField(
        blank=True, help_text=_('Text describing of this link.'),
        verbose_name=_('Description')
    )
    uuid = models.UUIDField(
        default=uuid.uuid4, editable=False, help_text=_(
            'Universally Unique ID. A unique identifier generated for each '
            'link.'
        ), verbose_name=_('UUID')
    )

    class Meta:
        #unique_together = ('uuid', 'document')
        verbose_name = _('Shareable link')
        verbose_name_plural = _('Shareable links')

    def get_api_image_url(self, document_page, *args, **kwargs):
        """
        Create an unique URL combining:
        - the page's image URL
        - the interactive argument
        - a hash from the server side and interactive transformations
        The purpose of this unique URL is to allow client side caching
        if document page images.
        """
        document_page = DocumentPage.objects.filter(
            document_version__document__id=self.document_id
        ).get(pk=document_page.pk)

        transformations_hash = BaseTransformation.combine(
            document_page.get_combined_transformation_list(*args, **kwargs)
        )

        kwargs.pop('transformations', None)

        final_url = furl()
        final_url.args = kwargs
        final_url.path = reverse(
            viewname='rest_api:shareablelink-documentpage-image', kwargs={
                'shareable_link_id': self.pk,
                'document_version_id': document_page.document_version.pk,
                'document_page_id': document_page.pk
            }
        )
        final_url.args['_hash'] = transformations_hash

        return final_url.tostr()

    def __str__(self):
        return str(self.uuid)
    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'event': event_shareable_link_created,
            'target': 'self',
        },
        edited={
            'event': event_shareable_link_edited,
            'target': 'self',
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)



from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.reverse import reverse

from mayan.apps.documents.serializers import DocumentSerializer

from .models import ShareableLink


def get_document_shareable_link_list_url(self, instance):
    return reverse(
        viewname='rest_api:document-shareablelink-list', kwargs={
            'document_id': instance.pk
        }, request=self.context['request'], format=self.context['format']
    )


DocumentSerializer._declared_fields['shareable_link_list_url'] = serializers.SerializerMethodField()#source='object_get_object_shareable_link_list_url')
DocumentSerializer.get_shareable_link_list_url = get_document_shareable_link_list_url
DocumentSerializer.Meta.fields += ('shareable_link_list_url',)


class DocumentShareableLinkSerializer(serializers.ModelSerializer):
    action_urls = serializers.SerializerMethodField()
    document = DocumentSerializer(read_only=True)
    url = serializers.SerializerMethodField()

    class Meta:
        fields = (
            'action_urls', 'description', 'document', 'id',
            'url', 'uuid'
        )
        model = ShareableLink

    def get_action_urls(self, instance):
        return {
            'document_download_url': reverse(
                viewname='shareable_links:shareable_link_document_download',
                kwargs={
                    'uuid': instance.uuid
                }, request=self.context['request'],
            ),
            'document_preview_url': reverse(
                viewname='shareable_links:shareable_link_document_preview',
                kwargs={
                    'uuid': instance.uuid
                }, request=self.context['request'],
            )
        }

    def get_url(self, instance):
        return reverse(
            'rest_api:document-shareablelink-detail', kwargs={
                'document_id': instance.document.pk,
                'shareable_link_id': instance.pk
            }, request=self.context['request'], format=self.context['format']
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Shareable link'), name='shareable_links'
)

event_shareable_link_created = namespace.add_event_type(
    label=_('Shareable link created'), name='shareable_link_created'
)
event_shareable_link_deleted = namespace.add_event_type(
    label=_('Shareable link deleted'), name='shareable_link_deleted'
)
event_shareable_link_edited = namespace.add_event_type(
    label=_('Shareable link edited'), name='shareable_link_edited'
)



from django.shortcuts import get_object_or_404

from mayan.apps.documents.api_views import APIDocumentPageImageView
from mayan.apps.documents.models import Document
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.rest_api import generics

from .models import ShareableLink
from .permissions import (
    permission_shareable_link_create, permission_shareable_link_delete,
    permission_shareable_link_edit, permission_shareable_link_view
)
from .serializers import DocumentShareableLinkSerializer


class APIDocumentShareableLinkListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the shareable link of the selected document.
    post: Create a new shareable link for the selected document.
    """
    mayan_object_permissions = {
        'GET': (permission_shareable_link_view,),
        'POST': (permission_shareable_link_view,),
    }
    serializer_class = DocumentShareableLinkSerializer

    def perform_create(self, serializer):
        serializer.validated_data['document'] = self.get_document()
        serializer.save()

    def get_document(self):
        return get_object_or_404(
            klass=Document, pk=self.kwargs['document_id']
        )

    def get_queryset(self):
        return self.get_document().shareable_links.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super().get_serializer(*args, **kwargs)


class APIDocumentShareableLinkDetailView(generics.RetrieveDestroyAPIView):
    """
    delete: Delete the selected shareable link.
    get: Return the details of the selected shareable link.
    """
    lookup_url_kwarg = 'shareable_link_id'
    mayan_object_permissions = {
        'DELETE': (permission_shareable_link_delete,),
        'GET': (permission_shareable_link_view,),
    }
    serializer_class = DocumentShareableLinkSerializer

    def get_document(self):
        return get_object_or_404(
            klass=Document, pk=self.kwargs['document_id']
        )

    def get_queryset(self):
        return self.get_document().shareable_links.all()

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super().get_serializer(*args, **kwargs)


class APIShareableLinkDocumentPageImageView(APIDocumentPageImageView):
    lookup_url_kwarg = 'document_page_id'

    def get_document_version(self):
        return get_object_or_404(
            self.get_document().versions.all(), pk=self.kwargs[
                'document_version_id'
            ]
        )

    def get_document(self):
        return self.get_shareable_link().document

    def get_shareable_link(self):
        return get_object_or_404(
            klass=ShareableLink, pk=self.kwargs['shareable_link_id']
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import get_cascade_condition

from .icons import (
    icon_shareable_link_delete, icon_shareable_link_edit,
    icon_shareable_link_list, icon_shareable_link_setup
)
from .permissions import (
    permission_shareable_link_create, permission_shareable_link_delete,
    permission_shareable_link_edit, permission_shareable_link_view
)

#link_shareable_link_backend_selection = Link(
#    icon_class=icon_shareable_link_backend_selection,
#    permissions=(permission_shareable_link_create,),
#    text=_('Create shareable link'),
#    view='shareable_links:shareable_link_backend_selection',
#)
link_shareable_link_delete = Link(
    args='resolved_object.pk',
    icon_class=icon_shareable_link_delete,
    permissions=(permission_shareable_link_delete,),
    tags='dangerous', text=_('Delete'),
    view='shareable_links:shareable_link_delete',
)
link_shareable_link_edit = Link(
    args='object.pk',
    icon_class=icon_shareable_link_edit,
    permissions=(permission_shareable_link_edit,),
    text=_('Edit'), view='shareable_links:shareable_link_edit',
)
link_shareable_link_list = Link(
    icon_class=icon_shareable_link_list,
    permissions=(permission_shareable_link_view,),
    text=_('Shareable link list'), view='shareable_links:shareable_link_list',
)
link_document_shareable_link_list = Link(
    args='resolved_object.pk', icon_class=icon_shareable_link_list,
    permissions=(permission_shareable_link_view,),
    text=_('Shareable links'),
    view='shareable_links:document_shareable_link_list',
)
link_shareable_link_setup = Link(
    condition=get_cascade_condition(
        app_label='shareable_links', model_name='SharebleLink',
        object_permission=permission_shareable_link_view,
        view_permission=permission_shareable_link_create,
    ), icon_class=icon_shareable_link_setup,
    text=_('Shareable links'),
    view='shareable_links:shareable_link_list'
)



from mayan.apps.appearance.classes import Icon

icon_shareable_links = Icon(
    driver_name='fontawesome', symbol='handshake'
)
icon_shareable_link_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='handshake',
    secondary_symbol='plus'
)
icon_shareable_link_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_shareable_link_edit = Icon(
    driver_name='fontawesome', symbol='pencil-alt'
)
icon_shareable_link_list = icon_shareable_links
icon_shareable_link_setup = icon_shareable_links



from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(
    label=_('Shareable link'), name='shareable_links'
)

permission_shareable_link_create = namespace.add_permission(
    label=_('Create shareablelinks'), name='shareable_link_create'
)
permission_shareable_link_delete = namespace.add_permission(
    label=_('Delete shareable links'), name='shareable_link_delete'
)
permission_shareable_link_edit = namespace.add_permission(
    label=_('Edit shareable links'), name='shareable_link_edit'
)
permission_shareable_link_view = namespace.add_permission(
    label=_('View shareable links'), name='shareable_link_view'
)



# Generated by Django 2.2.15 on 2020-12-20 07:59

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('documents', '0054_trasheddocument'),
    ]

    operations = [
        migrations.CreateModel(
            name='ShareableLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.TextField(help_text='Text describing of this link.', verbose_name='Description')),
                ('uuid', models.UUIDField(default=uuid.uuid4, help_text='Universally Unique ID. A unique identifier generated for each link.', verbose_name='UUID')),
                ('document', models.ForeignKey(help_text='The document to which to link.', on_delete=django.db.models.deletion.CASCADE, related_name='shareable_links', to='documents.Document', verbose_name='Document')),
            ],
            options={
                'verbose_name': 'Shareable link',
                'verbose_name_plural': 'Shareable links',
            },
        ),
    ]



import json

TEST_STORED_CREDENTIAL_BACKEND_PATH = 'credentials.credential_backends.OAuthAccessToken'
TEST_STORED_CREDENTIAL_BACKEND_DATA_FIELDS = {
    'access_token': 'access_token_data'
}
TEST_STORED_CREDENTIAL_BACKEND_DATA = json.dumps(
    obj=TEST_STORED_CREDENTIAL_BACKEND_DATA_FIELDS
)
TEST_STORED_CREDENTIAL_LABEL = 'test credential'
TEST_STORED_CREDENTIAL_LABEL_EDITED = 'test credential edited'



from django.conf.urls import url
from django.urls import path

from .api_views import (
    APIDocumentShareableLinkDetailView, APIDocumentShareableLinkListView,
    APIShareableLinkDocumentPageImageView
)
from .views import (
    DocumentShareableLinkListView, ShareableLinkDocumentDownloadView,
    ShareableLinkDocumentPreviewView, ShareableLinkDeleteView
)

urlpatterns = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/shareable_links/$',
        name='document_shareable_link_list',
        view=DocumentShareableLinkListView.as_view()
    ),
    #url(
    #    regex=r'^documents/(?P<document_id>\d+)/shareable_links/create/$',
    #    name='document_shareable_link_create',
    #    view=DocumentShareableLinkCreateView.as_view()
    #),
    url(
        regex=r'^shareable_links/(?P<shareable_link_id>\d+)/delete/$',
        name='shareable_link_delete',
        view=ShareableLinkDeleteView.as_view()
    ),
    #url(
    #    regex=r'^shareable_links/(?P<uuid>[a-zA-Z0-9_.]+)/use/$',
    #    name='shareable_link_use',
    #    view=ShareableLinkUseView.as_view()
    #),
    path(
        route='shareable_links/<uuid>/document/preview/',
        name='shareable_link_document_preview',
        view=ShareableLinkDocumentPreviewView.as_view()
    ),
    path(
        route='shareable_links/<uuid>/document/download/',
        name='shareable_link_document_download',
        view=ShareableLinkDocumentDownloadView.as_view()
    ),
]

api_urls = [
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/shareable_links/$',
        name='document-shareablelink-list',
        view=APIDocumentShareableLinkListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/shareable_links/(?P<shareable_link_id>[0-9]+)/$',
        name='document-shareablelink-detail',
        view=APIDocumentShareableLinkDetailView.as_view()
    ),



    #url(
    #    regex=r'^shareable_links/$', name='shareablelink-list',
    #    view=APIShareableLinkListView.as_view()
    #),
    #url(
    #    regex=r'^shareable_links/(?P<shareable_link_id>[0-9]+)/$',
    #    name='shareablelink-detail', view=APIShareableLinkDetailView.as_view()
    #),
    url(
        #regex=r'^shareable_links/(?P<shareable_link_id>[0-9]+)/documents/(?P<document_id>[0-9]+)/versions/(?P<document_version_id>[0-9]+)/pages/(?P<document_page_id>[0-9]+)/image/$',
        regex=r'^shareable_links/(?P<shareable_link_id>[0-9]+)/versions/(?P<document_version_id>[0-9]+)/pages/(?P<document_page_id>[0-9]+)/image/$',
        name='shareablelink-documentpage-image',
        view=APIShareableLinkDocumentPageImageView.as_view()
    ),
]



from django.contrib import admin

from .models import ShareableLink


@admin.register(ShareableLink)
class ShareableLinkAdmin(admin.ModelAdmin):
    list_display = ('document', 'uuid', 'description')



from django.db.models import Q

from ..models import StoredCredential

from .literals import (
    TEST_STORED_CREDENTIAL_BACKEND_DATA_FIELDS,
    TEST_STORED_CREDENTIAL_BACKEND_DATA, TEST_STORED_CREDENTIAL_BACKEND_PATH,
    TEST_STORED_CREDENTIAL_LABEL, TEST_STORED_CREDENTIAL_LABEL_EDITED
)


class StoredCredentialAPIViewTestMixin:
    def _request_test_stored_credential_create_api_view(self):
        pk_list = list(StoredCredential.objects.values_list('pk', flat=True))

        response = self.post(
            viewname='rest_api:credential-list', data={
                'label': TEST_STORED_CREDENTIAL_LABEL,
                'backend_path': TEST_STORED_CREDENTIAL_BACKEND_PATH,
                'backend_data': TEST_STORED_CREDENTIAL_BACKEND_DATA
            }
        )

        try:
            self.test_stored_credential = StoredCredential.objects.get(
                ~Q(pk__in=pk_list)
            )
        except StoredCredential.DoesNotExist:
            self.test_stored_credential = None

        return response

    def _request_test_stored_credential_delete_api_view(self):
        return self.delete(
            viewname='rest_api:credential-detail', kwargs={
                'credential_id': self.test_stored_credential.pk
            }
        )

    def _request_test_stored_credential_detail_api_view(self):
        return self.get(
            viewname='rest_api:credential-detail', kwargs={
                'credential_id': self.test_stored_credential.pk
            }
        )

    def _request_test_stored_credential_edit_via_patch_api_view(self):
        return self.patch(
            viewname='rest_api:credential-detail', kwargs={
                'credential_id': self.test_stored_credential.pk
            }, data={
                'label': '{} edited'.format(self.test_stored_credential.label)
            }
        )

    def _request_test_stored_credential_list_api_view(self):
        return self.get(viewname='rest_api:credential-list')


class StoredCredentialTestMixin(object):
    def _create_test_stored_credential(self):
        self.test_stored_credential = StoredCredential.objects.create(
            label=TEST_STORED_CREDENTIAL_LABEL,
            backend_path=TEST_STORED_CREDENTIAL_BACKEND_PATH,
            backend_data=TEST_STORED_CREDENTIAL_BACKEND_DATA
        )


class StoredCredentialViewTestMixin(object):
    def _request_test_stored_credential_backend_selection_view(self):
        return self.post(
            viewname='credentials:stored_credential_backend_selection', data={
                'backend': TEST_STORED_CREDENTIAL_BACKEND_PATH,
            }
        )

    def _request_test_stored_credential_create_view(self):
        data = {'label': TEST_STORED_CREDENTIAL_LABEL}
        data.update(TEST_STORED_CREDENTIAL_BACKEND_DATA_FIELDS)

        return self.post(
            viewname='credentials:stored_credential_create', kwargs={
                'class_path': TEST_STORED_CREDENTIAL_BACKEND_PATH,
            }, data=data
        )

    def _request_test_stored_credential_delete_view(self):
        return self.post(
            viewname='credentials:stored_credential_delete', kwargs={
                'stored_credential_id': self.test_stored_credential.pk
            }
        )

    def _request_test_stored_credential_edit_view(self):
        data = {'label': TEST_STORED_CREDENTIAL_LABEL_EDITED}
        data.update(TEST_STORED_CREDENTIAL_BACKEND_DATA_FIELDS)

        return self.post(
            viewname='credentials:stored_credential_edit', kwargs={
                'stored_credential_id': self.test_stored_credential.pk
            }, data=data
        )

    def _request_test_stored_credential_list_view(self):
        return self.get(viewname='credentials:stored_credential_list')



import logging

from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.urls import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _

from stronghold.views import StrongholdPublicMixin

from mayan.apps.common.generics import (
    FormView, SingleObjectDeleteView, SingleObjectCreateView,
    SingleObjectDownloadView, SingleObjectListView, SimpleView
)
from mayan.apps.common.mixins import ExternalObjectMixin
from mayan.apps.documents.models import Document
from mayan.apps.documents.settings import (
    setting_print_height, setting_print_width
)

from .icons import icon_shareable_links
#from .links import link_stored_shareable_link_backend_selection
from .models import ShareableLink
from .permissions import (
    permission_shareable_link_create, permission_shareable_link_delete,
    permission_shareable_link_edit, permission_shareable_link_view
)

logger = logging.getLogger(name=__name__)


class ShareableLinkDocumentDownloadView(
    StrongholdPublicMixin, SingleObjectDownloadView
):
    model = ShareableLink
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'

    def get_download_file_object(self):
        return self.object.document.open()

    def get_download_label(self):
        return force_text(self.object.document)


# StrongholdPublicMixin needs to be first
class ShareableLinkDocumentPreviewView(
    StrongholdPublicMixin, ExternalObjectMixin, SimpleView
):
    external_object_class = ShareableLink
    external_object_pk_url_kwargs = {'uuid': 'uuid'}
    template_name = 'shareable_links/document_preview.html'

    def get_extra_context(self):
        context = {
            'appearance_type': 'plain',
            'shareable_link': self.external_object,
            'pages': self.external_object.document.pages.all(),
            'width': setting_print_width.value,
            'height': setting_print_height.value,
        }

        return context


class ShareableLinkCreateView(SingleObjectCreateView):
    post_action_redirect = reverse_lazy(
        viewname='shareable_links:stored_shareable_link_list'
    )
    view_permission = permission_shareable_link_create

    def get_backend(self):
        try:
            return CredentialBackend.get(name=self.kwargs['class_path'])
        except KeyError:
            raise Http404(
                '{} class not found'.format(self.kwargs['class_path'])
            )

    def get_extra_context(self):
        return {
            'title': _(
                'Create a new shareable link.'
            ) % self.get_backend().label,
        }

    def get_form_schema(self):
        backend = self.get_backend()
        result = {
            'fields': backend.fields,
            'widgets': getattr(backend, 'widgets', {})
        }
        if hasattr(backend, 'field_order'):
            result['field_order'] = backend.field_order

        return result

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user,
            'backend_path': self.kwargs['class_path']
        }


class ShareableLinkDeleteView(SingleObjectDeleteView):
    model = ShareableLink
    object_permission = permission_shareable_link_delete
    pk_url_kwarg = 'shareable_link_id'
    post_action_redirect = reverse_lazy(
        viewname='shareable_links:stored_shareable_link_list'
    )

    def get_extra_context(self):
        return {
            'title': _('Delete shareable link: %s') % self.object,
        }


class DocumentShareableLinkListView(ExternalObjectMixin, SingleObjectListView):
    external_object_class = Document
    external_object_permission = permission_shareable_link_view
    external_object_pk_url_kwarg = 'document_id'
    object_permission = permission_shareable_link_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            #'hide_link': True,
            #'no_results_main_link': link_document_tag_multiple_attach.resolve(
            #    context=RequestContext(
            #        self.request, {'object': self.external_object}
            #    )
            #),
            'no_results_icon': icon_shareable_links,
            'no_results_text': _(
                'Shareable links give controlled access to document actions '
                'to external users.'
            ),
            'no_results_title': _('Document has no shareable links'),
            'object': self.external_object,
            'title': _(
                'Shareable links for document: %s'
            ) % self.external_object,
        }

    def get_source_queryset(self):
        return self.external_object.shareable_links.all()



# Generated by Django 2.2.15 on 2020-12-20 07:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shareable_links', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shareablelink',
            name='description',
            field=models.TextField(blank=True, help_text='Text describing of this link.', verbose_name='Description'),
        ),
    ]



from django.apps import apps
from django.shortcuts import get_object_or_404
from django.template import Library

register = Library()


@register.simple_tag
def shareable_links_documents_get_api_image_url(
    shareable_link, document_page, **kwargs
):
    #ShareableLink = apps.get_model(
    #    app_label='shareable_links', model_name='ShareableLink'
    #)
    #get_object_or_404(klass=hareableLink, pk=


    return shareable_link.get_api_image_url(document_page=document_page, **kwargs)



import logging

from django.apps import apps
from django.urls import reverse, reverse_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.links import link_acl_list
from mayan.apps.acls.permissions import permission_acl_edit, permission_acl_view
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_list_facet, menu_object, menu_secondary, menu_setup
)
from mayan.apps.events.classes import EventModelRegistry, ModelEventType
from mayan.apps.events.links import (
    link_events_for_object, link_object_event_types_user_subcriptions_list,
)
from mayan.apps.events.permissions import permission_events_view
from mayan.apps.navigation.classes import SourceColumn

from .events import event_shareable_link_edited
from .links import (
    link_document_shareable_link_list, link_shareable_link_delete,
    link_shareable_link_edit, link_shareable_link_list,
    link_shareable_link_setup
)
from .permissions import (
    permission_shareable_link_delete, permission_shareable_link_edit,
    permission_shareable_link_view
)

logger = logging.getLogger(name=__name__)


class ShareableLinkApp(MayanAppConfig):
    app_namespace = 'shareable_links'
    app_url = 'shareable_links'
    has_rest_api = True
    has_tests = True
    name = 'shareable_links'
    verbose_name = _('Shareable links')

    def ready(self):
        super().ready()

        ShareableLink = self.get_model(model_name='ShareableLink')

        Document = apps.get_model(app_label='documents', model_name='Document')

        EventModelRegistry.register(model=ShareableLink)

        ModelEventType.register(
            model=ShareableLink, event_types=(
                event_shareable_link_edited,
            )
        )

        SourceColumn(
            attribute='uuid', is_identifier=True, is_sortable=True,
            source=ShareableLink
        )
        #SourceColumn(
        #    attribute='description', include_label=True,
        #    source=ShareableLink
        #)
        SourceColumn(
            func=lambda context: reverse(
                viewname='shareable_links:shareable_link_use', kwargs={
                    'uuid': context['object'].uuid
                }
            ), label=_('URL'), source=ShareableLink
        )

        ModelPermission.register(
            model=ShareableLink, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_events_view, permission_shareable_link_delete,
                permission_shareable_link_edit, permission_shareable_link_view,
            )
        )

        menu_list_facet.bind_links(
            links=(
                link_document_shareable_link_list,
            ), sources=(Document,)
        )

        menu_list_facet.bind_links(
            links=(
                link_acl_list, link_events_for_object,
                link_object_event_types_user_subcriptions_list,
            ), sources=(ShareableLink,)
        )

        menu_object.bind_links(
            links=(
                link_shareable_link_delete,
                #link_shareable_link_edit,
            ), sources=(ShareableLink,)
        )

        menu_secondary.bind_links(
            links=(
                #link_shareable_link_create,
                link_shareable_link_list,
            ), sources=(
                ShareableLink,
                'shareable_links:stored_shareable_link_create',
                'shareable_links:stored_shareable_link_list'
            )
        )

        #menu_setup.bind_links(
        #    links=(link_shareable_link_setup,)
        #)
